<?php
namespace Fiscal\Logic\NotaFiscal\Entrada;

use Estoque\Entities\Posestdia;
use Fiscal\Entities\Notafiscal;
use Fiscal\Logic\NotaFiscal\Exceptions\NFEntradaException;
use Fiscal\Logic\NotaFiscal\Exceptions\NFSaidaException;
use Geral\Entities\Itemproduto;
use Geral\Entities\Paramfilial;
use Geral\Entities\Prodfili;
use Geral\Entities\Tipooperac;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\TratamentoDeMensagens;

/**
 * Created by JetBrains PhpStorm.
 * User: maximiliansilva
 * Date: 22/01/13
 * Time: 15:06
 * To change this template use File | Settings | File Templates.
 */
class NFValidaItemproduto{

    private $itemproduto;

    public function __construct(\Geral\Entities\Itemproduto $itemproduto){
        $this->itemproduto = $itemproduto;
    }

    /**
     * Valida produto inativo e nao cadastrado
     *
     * @static
     * @return string
     * @author maximiliansilva 22/01/13
     */
    public function validaProdutoInativo(){
        if (is_object($this->itemproduto->getProdfili()) &&
            $this->itemproduto->getProdfili()->getIdprodativ() !== \Geral\Entities\Prodfili::PRODUTO_ATIVO
        ){

            NFEntradaException::produtoInativoFilial();
        }

        return true;
    }

    /**
     * Valida se o produto informado está relacionado para a filial
     * @return void
     */
    public function validaProdutoFilial(){
        if(!$this->itemproduto->getProdfili()){
            NFEntradaException::produtoNaoRelacionadoFilial();
        }
    }

    public function precoProdutoIrregular(){

        $paramestoque = \Geral\Entities\Paramestoque::findOneByNrorg(\Geral\Util\Ambiente::getNrorg());
        $itemproduto  = $this->itemproduto;
        $notafiscal   = $itemproduto->getLanctonf();

        if ($itemproduto->getCdproduto()){

            if ($notafiscal->getCdtipooper() != '03' && $paramestoque->getVrpetoleentr() && $itemproduto->getQtitemprod() > 0){

                if ($itemproduto->getProduto()->getVrfatoconv() === '0' || is_null($itemproduto->getProduto()->getVrfatoconv())){
                    $vrfatconv = 1;
                }
                else{
                    $vrfatconv = $itemproduto->getProduto()->getVrfatoconv();
                }

                $custoCmpProd = \Estoque\Entities\Custocmpprod::findOneByNrorgAndCdfilialAndCdproduto(
                    $itemproduto->getNrorg(),
                    $itemproduto->getCdfilial(),
                    $itemproduto->getCdproduto()
                );

                if (is_object($custoCmpProd)){
                    $vritem       = ((($itemproduto->getVrtotaitem() - $itemproduto->getVrdescitem()) / $itemproduto->getQtitemprod()) / $vrfatconv);
                    $vrmaior      = $vritem + ($vritem * $paramestoque->getVrpetoleentr() / 100);
                    $vrmenor      = $vritem - ($vritem * $paramestoque->getVrpetoleentr() / 100);
                    $vrliqcmpprod = $custoCmpProd->getVrliqcmpprod();
                    if ($vrliqcmpprod != 0){
                        if (($vrliqcmpprod > $vrmaior) || ($vrliqcmpprod < $vrmenor)){

                            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('msg_preco_produto_irregular');
                            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('msg_ultima_compra',
                                array($vrliqcmpprod));
                            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('msg_valor_do_produto',
                                array($vritem));
                            throw new \Exception(\Geral\Util\AcumuladorDeMensagens::ID_ACUMULADOR_DE_MENSAGENS);
                        }
                    }
                }
            }
        }

        return true;
    }

    public static function validaQtItemDevolucaoAoInserir($quantidade_antiga, $quantidade, $quantidade_resta_devolver,
                                                   $qtJaDevolvida){
        if ($quantidade_antiga == 0 && $quantidade > $quantidade_resta_devolver){
            return NFEntradaException::qtItemDevolverMenorIgualQtOriginal($qtJaDevolvida);
        }
        return true;
    }

    public static function validaQtItemDevolucaoAoAlterar($lancamento, $filialOrigem, $lancamentoOrigem, $nrdecitotanf,
                                                   $cdarvprod, $quantidade, $quantidade_origem, $quantidade_antiga,
                                                   $quantidade_origem, $qtJaDevolvida, $produto){
        $qtitemdevo = Itemproduto::getRepository()->consultaQuantidadeJaDevolvida($lancamento, $filialOrigem,
            $lancamentoOrigem, $nrdecitotanf, $produto, $cdarvprod);

        if ($qtitemdevo == 0 && $quantidade > $quantidade_origem){
            return NFEntradaException::qtItemDevolverMenorIgualQtOrigem();
        }
        elseif ($quantidade + $qtitemdevo > $quantidade_origem){
            return NFEntradaException::qtItemDevolverMenorIgualQtOriginal($qtJaDevolvida);
        }
        return true;
    }

    public function quantidadeProdutoEstoque(Tipooperac $tipoOperacao, $quantidadeAntiga){
        $cdalmoxarife = $this->itemproduto->getCdalmoxarife() ? : ' ';
        $cdlocalestoq = $this->itemproduto->getCdlocalestoq() ? : ' ';
        $nrloteestq   = $this->itemproduto->getNrloteestq() ? : ' ';
        $nrsublote    = $this->itemproduto->getNrsublote() ? : ' ';

        if ($this->itemproduto->getQtitemprod() <= 0 && $tipoOperacao->getIdoperdevonf() != 'S' && $tipoOperacao->getCdtipooper() != '03' && $tipoOperacao->getIdnfajuste() != 'S')
            NFSaidaException::quantidadeDeveSerMaiorQueZero();

        if ($tipoOperacao->getCdtipooper() != '03' && $tipoOperacao->getIdatuesttpop() != 'N'){
            /* @var $notafiscal Notafiscal */
            $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($this->itemproduto->getCdfilial(),
                $this->itemproduto->getNrlanctonf(), $this->itemproduto->getNrorg());
            if ($notafiscal->getIddocfatur() != 'S'){
                /* @var $paramfilial Paramfilial */
                $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($this->itemproduto->getCdfilial(),
                    $this->itemproduto->getNrorg());
                if ($paramfilial->getIdctrlestq() != 'N' && $paramfilial->getIdestonline() != 'N'){
                    /* @var $prodfili Prodfili */
                    $prodfili = Prodfili::findOneByCdprodutoAndCdfilialAndNrorg($this->itemproduto->getCdproduto(),
                        $this->itemproduto->getCdfilial(),
                        $this->itemproduto->getNrorg());
                    if ($prodfili->getIdcntrestoq() == 'S'){
                        $dtposiest = Posestdia::getRepository()->retornaDataPosicaoEstoque(Ambiente::getCdFilial(),
						    $cdalmoxarife,
                            $cdlocalestoq, $nrloteestq, $nrsublote, $this->itemproduto->getCdproduto(),
                            DateUtil::adicionaIntervalo($notafiscal->getDtentrsaid())->format(DateUtil::FORMATO_BRASILEIRO));
						
                        $tipoCusto = $tipoOperacao->getCdtipocusto();
                        if (!$tipoCusto){
                            /* @var $paramfilial Paramfilial */
                            $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($this->itemproduto->getCdfilial(),
                                $this->itemproduto->getNrorg());
                            $tipoCusto   = $paramfilial->getIdtcusnfroma() ? : 3;
                        }

                        $qtestoqdia = Posestdia::getRepository()->retornaCustoProdutoEstoque(Ambiente::getCdFilial(),
                            $tipoCusto, $notafiscal->getDtentrsaid()->format(DateUtil::FORMATO_BRASILEIRO),
                            $cdalmoxarife,
                            $cdlocalestoq, $this->itemproduto->getCdproduto(), $nrloteestq, $nrsublote, $dtposiest, -1);

                        $quantidadeDisponivel = ($quantidadeAntiga + $qtestoqdia[0]['QTESTOQDIA']);
                        if ($this->itemproduto->getQtitemprod() > $quantidadeDisponivel && $tipoOperacao->getIdoperdevonf() != 'S' && $paramfilial->getIdpermvisposest() != 'N' && $paramfilial->getIdpermestneg() != 'S')
                            NFSaidaException::quantidadeInformadaNaoDisponivelNoEstoque($quantidadeDisponivel);
                    }
                }
            }
        }
    }
}
